DAT fileName  byte "ButtonMenu111212a", 0
' I like to keep the program title in a top DAT section like this
' so I can have it displayed with a serial terminal.
' I frequently forget what program is loaded into a Propeller board.
' By having the program display the fileName, I'm reminded of
' which program and which version is loaded in the Propeller.
CON
{{  Public Notes

  This program demonstrates using the touchpads
  on a QuickStart board to select which methods
  of a program to execute.
  Most of the methods may be exited by pressing
  pad 0 (far right pad) on the QuickStart board.

  The methods in this program only flash the
  boards LEDs.  These methods are intended to
  illustrate some of the ways to exit a method.

  The exit strategies illustrated are:
  (Method7)
  A "repeat while" loop that checks
  a condition once per loop.  If the condition
  is true the loop repeats.  If the condition
  is not met, the program flow returns to the
  place in code immediately after the call to
  the method.
  (Method6, Method4 & Method3)
  One or more "abort" statements returns
  the flow of the program to immediately after
  the call to the method with an abort trap
  (see page 48 of Propeller Manual v1.2 for
  more information about aborts and abort
  traps).
  (Method5)
  The method must finished its programmed
  execution before the flow of the program
  returns to the place in code immediately
  after the call to the method.

  None of these exit strategies return a value
  (other than the default zero).
  
}}
{  I'm not sure who wrote TouchButtons111029a.
  I think it may be my adaptation of the
  original QuickStart demo program.
  This object works well in my climate
  and with my fingers.  I've read the QS boards
  can be picky about finger moisture.
  YMMV.
}                                                    
CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

  _ExitPad = 0 ' used to end some methods
  
   
VAR

  long  delay
'' The variable "delay" is used by the "Buttons" object
'' to indicate how much time a pad's voltage should be allowed
'' to decay prior to testing its state (to see if pad has been pressed).

  byte  ledByte, button[8]
  
'' The variable "ledByte" makes it easier to display (with LEDs)
'' which buttons have been pressed.  The button states are held
'' in this single byte so when used with "outa[23..16]" the LEDs
'' corresponding to each pad will be lit.
''
'' The array "button" holds the current pad states.
'' Examples: button[0] will equal 1 while pad zero (far right pad)is pressed.
''           button[7] will equal 0 while pad seven (far left pad) is not pressed.
''

OBJ
 
  Buttons : "TouchButtons111029a"
'' The object "Buttons" fills the array "button" with ones or zeros corresponding
'' to the state (whether or not it is pressed) of each pad.
''
PUB Setup
'' The program will start with this method "Setup" since it is the first public
'' method.
'' I called it "Setup" since this is where things that need to happen just once
'' occur.  (Setup or Init is a common name for this type of method.)
''
 
  delay := clkfreq / 100          ' used by Buttons object
'Set the varible delay to the number of clock cycles in one hundreth of a second.
'' At 80MHz (the QuickStart's default clock speed), this will be 800,000 clock cycles.

  Buttons.start(@delay, @button)
' Launch the touch buttons driver sampling 100 times a second.
' The driver (or object) will store the pad states in the array "button".
 
  waitcnt(clkfreq/1_000 + cnt)    ' give the cog a little time to launch
  dira[23..16]~~                  ' Set the LEDs as outputs
                   
  MainLoop 'Call MainLoop method
   
    
PUB MainLoop | localIndex
'' MainLoop is likely where the program will speed most of its time.
'' 
  repeat
    ledByte := 0
' "ledByte" makes it easier to display, with LEDs, which buttons have been pressed.
'
    repeat localIndex from 7 to 0
      ledByte <<= 1
      ledByte += button[localIndex]
    outa[23..16] := ledByte
    
    if button[7] == 1
      waitcnt(clkfreq / 4 + cnt) ' wait a bit so the button press will register on the LED 
      Method7
'' Touching pad 7 (far left pad) will start Method7.
'' Pads 6 through 3 also launch corresponding methods.
'' Touching pads 2 through 0 will only light their corresponding LED.
'' Pad 0 (far right pad) will return the flow of the program back to
'' this method ("MainLoop") from many of the other methods (all but
'' Method5).
''     
    if button[6] == 1
      waitcnt(clkfreq / 4 + cnt) ' wait a bit so the button press will register on the LED
      \Method6  ' "\" is an abort trap
    if button[5] == 1 
      waitcnt(clkfreq / 4 + cnt) ' wait a bit so the button press will register on the LED
      Method5
    if button[4] == 1 
      waitcnt(clkfreq / 4 + cnt) ' wait a bit so the button press will register on the LED
      \Method4
    if button[3] == 1 
      waitcnt(clkfreq / 4 + cnt) ' wait a bit so the button press will register on the LED
      \Method3
      
PUB Method7
'' Method7 only checks if it should exit between the LEDs flashing sets.
'' (About once every four seconds.)
'' Holding the "_ExitPad" (pad 0) will trigger the exit.
''
  repeat
    ' Add code here
    ' example code to flash LEDs
    
    waitcnt(clkfreq / 2 + cnt)
    repeat 7
      outa[23..16] := %1111_1111 ' turn all the LEDs on
      waitcnt(clkfreq / 4 + cnt)
      outa[23..16] := 0          ' turn all the LEDs off
      waitcnt(clkfreq / 4 + cnt)
      
  while button[_ExitPad] == 0
'' The program flow returns to "MainLoop" only if "_ExitPad" (pad 0) is held
'' while the pad's state is checked (with the line "while button[_ExitPad] == 0"
'' at the bottom of the loop.
''
PUB Method6
'' Method6 ends with an abort.
'' The method check for the abort state frequently
'' (About four times a second.)
'' Pressing the "_ExitPad" (pad 0) initiates the abort.
''
  repeat
    ' Add code here
    ' example code to flash LEDs
    if button[_ExitPad] == 1
      abort
    waitcnt(clkfreq / 4 + cnt)
    if button[_ExitPad] == 1
      abort
    waitcnt(clkfreq / 4 + cnt)
    if button[_ExitPad] == 1
      abort
    repeat 6
      outa[23..16] := %1111_1111 ' turn all the LEDs on
      waitcnt(clkfreq / 4 + cnt)
      if button[_ExitPad] == 1
        abort
      outa[23..16] := 0          ' turn all the LEDs off
      waitcnt(clkfreq / 4 + cnt)
      if button[_ExitPad] == 1
        abort
'' Any of the above aborts returns the flow of the program back to "MainLoop".
''
PUB Method5
'' Method5 will only exit after it executes five sets of LEDs flashing.
'' There is no operator interaction.
'' This method takes about 15 seconds to complete.
''
  repeat 5
    ' Add code here
    ' example code to flash LEDs
    
    waitcnt(clkfreq / 2 + cnt)
    repeat 5
      outa[23..16] := %1010_1010 ' turn every other LED on
      waitcnt(clkfreq / 4 + cnt)
      outa[23..16] := %0101_0101 ' turn off the four that were on 
                                 ' and turn on the ones that were off
      waitcnt(clkfreq / 4 + cnt)
      
  outa[23..16] := 0
'' By reaching the end of the method, the program flow
'' automatically returns to the place in code immediately after
'' the call to this method (line 100).
PUB Method4 | localPattern ' declare a local variable named "localPattern"
'' Method4 ends with an abort.
'' The method checks for the abort state frequently.
'' (About 8 times a second.)
'' Pressing the "_ExitPad" (pad 0) initiates the abort.
'' This method rotates (to the left) a pattern across the LEDs.
''

  localPattern := %0001_0001_0001_0001_0001_0001_0001_0001
  ' The local variable "localPattern" is used to rotate
  ' an on and off pattern across the LEDs.
  ' It's a good idea to only use the rotate operator with longs.
  ' All local variables are longs.
  ' All local variables (except result) need to initialized.
  
  repeat ' repeats until the "abort" is called
    ' Add code here
    ' example code to flash LEDs
    
    waitcnt(clkfreq / 8 + cnt)
    outa[23..16] := localPattern
    localPattern <-= 1   ' Rotate "localPattern" on bit to the left.
                         ' The rotate operator brings the far left bit
                         ' around to the far right bit.
    if button[_ExitPad] == 1
      abort              
'' This "abort" returns the flow of the program back to "MainLoop".
''
PUB Method3 | localPattern
'' Method3 ends with an abort
'' The method checks for the abort state frequently.
'' (About 8 times a second.)
'' Pressing the "_ExitPad" (pad 0) initiates the abort.
'' This method rotates (to the right) a pattern across the LEDs.
''

  localPattern := %0001_0001_0001_0001_0001_0001_0001_0001
  ' See Method4 for explaination about "localPattern".
  
  repeat
    ' Add code here
    ' example code to flash LEDs
    
    waitcnt(clkfreq / 8 + cnt)
    outa[23..16] := localPattern
    localPattern ->= 1   ' Rotate "localPattern" on bit to the right.
    if button[_ExitPad] == 1
      abort

{{  Methods 2 and 1 are left as an exercise for the reader.}}  
'---[End of Program]-----------------------------------------------------------